// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// Damp.cpp - functions and macros for logging

/**
 *  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
 *  Pieter de Zwart <pdezwart@scu.edu>, David Mart <David.Mart@btinternet.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   In addition, as a special exception, <name of copyright
 *   holder> gives permission to link the code of this program with
 *   the FOO library (or with modified versions of FOO that use the
 *   same license as FOO), and distribute linked combinations including
 *   the two.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than FOO.  If you modify
 *   this file, you may extend this exception to your version of the
 *   file, but you are not obligated to do so.  If you do not wish to
 *   do so, delete this exception statement from your version.
 *
**/

/**
 * This file is part of Damage Amp.
 *
 *   Log Daemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Damp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without eveninjure the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Damage Amp; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

#include <extdll.h>		// always
#include <sdk_util.h>
#include <meta_api.h>
#include <cbase.h>

#include "Damp.h"
#include "DampRule.h"
#include "RuleCollection.h"
#include "DampDebug.h"
#include "amutil.h"


DampRule* defaultRule;
RuleCollection* ruleBook;

//prototypes for "private" functions
void resolveDamage( int iAttacker, int iVictim, int damage );
BOOL forceDeath( CBaseEntity* player );
void notifyClients( int type, int uid );

void injureLocal(void)
{
    LOG_DEBUG( 1, ("InjureLocal Called: Params<%s>", CMD_ARGS()) );
    int p1 = atoi( CMD_ARGV(1) );
    int p2 = atoi( CMD_ARGV(2) );
    
    //find the location of the damage
    const char* params = CMD_ARGS();
    int loc = 0;
    while( params[ loc ] != ':' )
    {
        loc++;
    }
    loc += 1; //starting of the number
    int loc2 = loc;
    //find the next space
    while( params[ loc2 ] != ' ' )
    {
        loc2++;
    }
   
    //parse out the damage
    char* strNum =  new char[ 4 ];
    int len = loc2 - loc;
    if( len > 3 )
        len = 3;    //make sure we fit in the array
    int x = 0;
    for(; x < len; x++ )
    {
        strNum[ x ] = params[ loc + x ];
    }
    strNum[  x ] = 0;   //null terminate the char array

    int damage = atoi( strNum );
    LOG_DEBUG( 1, ("Injury Recieved: Att<%i> Vic<%i> DamageStr<%s> DamageInt<%i>", p1, p2, strNum, damage) );
    resolveDamage( p1, p2, damage );
    delete strNum;
}


void setDefaultRule(void)
{
    //needs 8 args;
    if( CMD_ARGC() != 9 )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_def_rule>: 8 arguments not given" );
        return;
    }

    float m1 = atof( CMD_ARGV(1) );
    float m2 = atof( CMD_ARGV(2) );
    float m3 = atof( CMD_ARGV(3) );
    float m4 = atof( CMD_ARGV(4) );
    int d1 = atoi( CMD_ARGV(5) );
    int d2 = atoi( CMD_ARGV(6) );
    int d3 = atoi( CMD_ARGV(7) );
    int d4 = atoi( CMD_ARGV(8) );

    defaultRule->setModifiers( m1, m2, m3, m4, d1, d2, d3, d4 );
    notifyClients( DAMP_DEFAULT, 0 );
}

void addRule(void)
{
    if( CMD_ARGC() != 10 )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_add_rule>: 9 arguments not given" );
        return;
    }
    int index = GetPlayerIndex( CMD_ARGV(1) );
    if( index == 0 )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_add_rule>: Invalid UID \"%s\"", CMD_ARGV(1) );
        return;
    }
	CBaseEntity *person = UTIL_PlayerByIndex( index );
    if( !IsPlayerValid( person ) )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_add_rule>: Invalid Player with UID \"%s\"", CMD_ARGV(1) );
        return;
    }


    float m1 = atof( CMD_ARGV(2) );
    float m2 = atof( CMD_ARGV(3) );
    float m3 = atof( CMD_ARGV(4) );
    float m4 = atof( CMD_ARGV(5) );
    int d1 = atoi( CMD_ARGV(6) );
    int d2 = atoi( CMD_ARGV(7) );
    int d3 = atoi( CMD_ARGV(8) );
    int d4 = atoi( CMD_ARGV(9) );

    DampRule* rule = new DampRule( m1, m2, m3, m4, d1, d2, d3, d4 );
    ruleBook->addRule( person, rule );    
    notifyClients( DAMP_PERSONAL, GETPLAYERUSERID( person->edict() ) );
}

void delRule(void)
{
    if( CMD_ARGC() != 2 )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_del_rule>: 1 argument not given" );
        return;
    }
    int index = GetPlayerIndex( CMD_ARGV(1) );
    if( index == 0 )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_del_rule>: Invalid UID \"%s\"", CMD_ARGV(1) );
        return;
    }
	CBaseEntity *person = UTIL_PlayerByIndex( index );
    if( !IsPlayerValid( person ) )
    {
        LOG_MESSAGE( PLID, "ERROR<damp_del_rule>: Invalid Player with UID \"%s\"", CMD_ARGV(1) );
        return;
    }
    if( ruleBook->delRule( person ) )
    {
        notifyClients( DAMP_PERSONAL, GETPLAYERUSERID( person->edict() ) );
    }
}

void printLine( edict_t *pEntity, const char* msg )
{
    if( pEntity == 0 )
    {
        LOG_CONSOLE( PLID, msg );
    }
    else
    {
        char temp[ 256 ];
        strcpy( temp, msg );
        strcat( temp, "\n" );
        CLIENT_PRINTF( pEntity, print_console, temp );
    }
}


void printStatus( edict_t *pEntity )
{

    DampRule* rule;
    if( CMD_ARGC() > 2 )
    {
        printLine( pEntity, "This function takes no arguments (default rule) or 1 argument (uid of player)" );
    }
    else if( CMD_ARGC() == 2 )
    {
        int index = GetPlayerIndex( CMD_ARGV(1) );
        if( index == 0 )
        {
            printLine( pEntity, UTIL_VarArgs("ERROR<damp_status>: Invalid UID \"%s\"", CMD_ARGV(1)) );
            return;
        }
	    CBaseEntity *person = UTIL_PlayerByIndex( index );

	    if ( !IsPlayerValid(person) )
	    {
	        printLine( pEntity, UTIL_VarArgs("ERROR<damp_status>: Invalid Player, cannot take action to modifiy damage") );
	        return;
        }

        DampRule* temp;
        if( !( temp = ruleBook->findByPerson( person ) ) )
	    {
	        printLine( pEntity, UTIL_VarArgs("[Damp]: No Rule for player with UID \"%i\"", GETPLAYERUSERID( person->edict() )) );
	        return;
        }
        rule = temp;
        printLine( pEntity, UTIL_VarArgs("\nDamp Rule Status: UID %i", GETPLAYERUSERID( person->edict() )) );
    }
    else
    {
        rule = defaultRule;
        printLine( pEntity, "\nDamp Rule Status: Default Rule" );
    }

    printLine( pEntity, UTIL_VarArgs("%-12s %-12s %-12s %-12s %-12s", "", "Attacker", "Victim", "Att-FDeath", "Vic-FDeath" ) );

    float m1 = rule->getModifier( DAMP_OPP, DAMP_ATT );
    float m2 = rule->getModifier( DAMP_OPP, DAMP_VIC );
    int b1 = rule->getDeathMod( DAMP_OPP, DAMP_ATT );
    int b2 = rule->getDeathMod( DAMP_OPP, DAMP_VIC );

    printLine( pEntity, UTIL_VarArgs("%-12s %-12f %-12f %-12i  %-12i", "Opp", m1, m2, b1, b2) );

    m1 = rule->getModifier( DAMP_TEAM, DAMP_ATT );
    m2 = rule->getModifier( DAMP_TEAM, DAMP_VIC );
    b1 = rule->getDeathMod( DAMP_OPP, DAMP_ATT );
    b2 = rule->getDeathMod( DAMP_OPP, DAMP_VIC );

    printLine( pEntity, UTIL_VarArgs("%-12s %-12f %-12f %-12i  %-12i", "Team", m1, m2, b1, b2) );

}

void handleServerStatus(void)
{
    printStatus( 0 );
}

void handleClientCommand( edict_t *pEntity )
{
    if( strcmp( CMD_ARGV(0), "damp_status" ) == 0 )
    {
        printStatus( pEntity );
        RETURN_META(MRES_SUPERCEDE);
    }
    RETURN_META(MRES_HANDLED);
}

void dampInit(void)
{
	//LOG_DEBUG( 1, ( "Initializing Damp Core" ) );
    LOG_MESSAGE( PLID, "Initializing Damp Core" );
    REG_SVR_COMMAND("damp_status", handleServerStatus);
    REG_SVR_COMMAND("damp_injurelocal", injureLocal);
    REG_SVR_COMMAND("damp_def_rule", setDefaultRule);
    REG_SVR_COMMAND("damp_add_rule", addRule);
    REG_SVR_COMMAND("damp_del_rule", delRule);
}
void dampActivate(void)
{
    LOG_MESSAGE( PLID, "Activating Damp Core" );
    //create the necessary structures
    defaultRule = new DampRule( 0.0, 0.0, 0.0, 0.0, FALSE, FALSE, FALSE, FALSE );
    ruleBook = new RuleCollection( defaultRule );

    //get LogD to notify us
    SERVER_COMMAND( "logd_reg 58 damp_injurelocal\n" );
}

void dampDeactivate(void)
{
    LOG_MESSAGE( PLID, "Deactivating Damp Core" );
    //delete everything to get a fresh slate
    delete ruleBook;
    delete defaultRule;
}










void resolveDamage( int iAttacker, int iVictim, int damage )
{
    BOOL safe;

    int dType;

    float aMod;
    float vMod;
    int health;

    DampRule* aRule = NULL;
	int aTeam;

	DampRule* vRule = NULL;
	int vTeam;

	DampRule* activeRule = NULL;

	CBaseEntity *pAttacker = UTIL_PlayerByIndex( iAttacker );
	CBaseEntity *pVictim = UTIL_PlayerByIndex( iVictim );


	if ( !IsPlayerValid(pAttacker) || !IsPlayerValid(pVictim) )
	{
	    LOG_MESSAGE( PLID, "ERROR: Invalid Players, cannot take action to modifiy damage" );
	    return;
    }

    //figure out what the active rule is going to be
    if( !( aRule = ruleBook->findByPerson( pAttacker ) ) )
	{
	    LOG_MESSAGE( PLID, "ERROR: No Rule for Attacker> UID = %i", GETPLAYERUSERID( pAttacker->edict() ) );
	    return;
    }

    if( !( vRule = ruleBook->findByPerson( pVictim ) ) )
	{
	    LOG_MESSAGE( PLID, "ERROR: No Rule for Victim> UID = %i", GETPLAYERUSERID( pVictim->edict() ) );
	    return;
    }

    //higher id's have higher priorities
    if( aRule->id > vRule->id )
    {
        LOG_DEBUG( 1, ("Attacker Rules Chosen: A-%i vs V-%i", aRule->id, vRule->id) );
        activeRule = aRule;
    }
    else //in case of tie or victim has higher priority, victim rules apply
    {
        LOG_DEBUG( 1, ("Victim Rules Chosen: A-%i vs V-%i", aRule->id, vRule->id) );
        activeRule = vRule;
    }


    //get teams
    aTeam = get_player_team( pAttacker );
    vTeam = get_player_team( pVictim );
    LOG_DEBUG( 2, ( "Attacker Team: %-10i Victim Team: %i", aTeam, vTeam ) );
    if( aTeam == vTeam )
        dType = DAMP_TEAM;
    else
        dType = DAMP_OPP;


    aMod = activeRule->getModifier( dType, DAMP_ATT );
    vMod = activeRule->getModifier( dType, DAMP_VIC );

    // ****************
    //  perform rules
    // ****************
    LOG_DEBUG( 1, ("Performing Rules") );
    //LOG_DEBUG( 1, ( "Performing Rules" ) );

    //victim first
    safe = TRUE;
    health = pVictim->edict()->v.health;    //use 'health' so we don't mess with health directly and screw something up
    LOG_DEBUG( 1, ("Victim Status: Saved Health<%i>", health) );
    health += (int)( vMod * damage );
    LOG_DEBUG( 1, ("Victim Status: Damage<%i> Modifier<%f> Modified Health<%i>", damage, vMod, health) );

    //do value checking
    if( health > 100 )
    {
        LOG_DEBUG( 2, ("Victim Status: Health is greater than 100<%i>, setting to 100", health) );
        health = 100;
    }
    else if ( health < 0 )
    {
        LOG_DEBUG( 1, ("Victim Status: Health is less than 0<%i>, setting to 1", health) );
        health = 1; //don't kill the person ourselves

        //see if we need to force kill
        if( activeRule->getDeathMod( dType, DAMP_VIC ) == TRUE )
        {
            LOG_DEBUG( 1, ("Victim Status: Needs to die") );
            safe = FALSE;
            if( !forceDeath( pVictim ) )
            {
        	    LOG_MESSAGE( PLID, "ERROR: Could not force Victim's death> UID = %i", GETPLAYERUSERID( pVictim->edict() ) );
            }
        }
    }
    //health is safe now
    if( safe )
    {
        LOG_DEBUG( 2, ("Victim Status: Health after value checking<%i>", health) );
        pVictim->edict()->v.health = health;
    }
    else
    {
        LOG_DEBUG( 1, ("Victim Status: Health health operation not safe") );
    }

    //now the attacker
    safe = TRUE;
    health = pAttacker->edict()->v.health;    //use vH so we don't mess with health directly and screw something up
    LOG_DEBUG( 1, ("Attacker Status: Saved Health<%i>", health) );
    health += (int)( aMod * damage );
    LOG_DEBUG( 1, ("Attacker Status: Damage<%i> Modifier<%f> Modified Health<%i>", damage, aMod, health) );

    //do value checking
    if( health > 100 )
    {
        LOG_DEBUG( 2, ("Attacker Status: Health is greater than 100<%i>, setting to 100", health) );
        health = 100;
    }
    else if ( health < 0 )
    {
        LOG_DEBUG( 1, ("Attacker Status: Health is less than 0<%i>, setting to 1", health) );
        health = 1; //don't kill the person ourselves

        //see if we need to force kill
        if( activeRule->getDeathMod( dType, DAMP_ATT ) == TRUE )
        {
            LOG_DEBUG( 1, ("Attacker Status: Needs to die") );
            safe = FALSE;
            if( !forceDeath( pVictim ) )
            {
        	    LOG_MESSAGE( PLID, "ERROR: Could not force Attacker's death> UID = %i", GETPLAYERUSERID( pAttacker->edict() ) );
            }
        }
    }
    //health is safe now
    if( safe )
    {
        LOG_DEBUG( 2, ("Attacker Status: Health after value checking<%i>", health) );
        pAttacker->edict()->v.health = health;
    }
    else
    {
        LOG_DEBUG( 1, ("Attacker Status: Health health operation not safe") );
    }
}

BOOL forceDeath( CBaseEntity* player )
{
    return FALSE;
}

void notifyClients( int type, int uid )
{
    if( type == DAMP_PERSONAL )
    {
	    UTIL_ChatMsgAll( UTIL_VarArgs("[Damp] Personal Rules Changed for UID \"%i\"", uid) );
	    UTIL_ChatMsgAll( UTIL_VarArgs("[Damp] Type \"damp_status %i\" in console for info", uid) );
    }
    else
    {
	    UTIL_ChatMsgAll( "[Damp] Default Rules Changed" );
	    UTIL_ChatMsgAll( "[Damp] Type \"damp_status\" in console for info" );
    }
}

