// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// RuleCollection.h - keeps track of rules

/**
 *  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
 *  Pieter de Zwart <pdezwart@scu.edu>, David Mart <David.Mart@btinternet.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   In addition, as a special exception, <name of copyright
 *   holder> gives permission to link the code of this program with
 *   the FOO library (or with modified versions of FOO that use the
 *   same license as FOO), and distribute linked combinations including
 *   the two.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than FOO.  If you modify
 *   this file, you may extend this exception to your version of the
 *   file, but you are not obligated to do so.  If you do not wish to
 *   do so, delete this exception statement from your version.
 *
**/

/**
 * This file is part of Damage Amp.
 *
 *   Log Daemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Log Daemon is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Damage Amp; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

#include <extdll.h>
#include <eiface.h> // for CBase
#include <sdk_util.h>
#include <meta_api.h>
#include <cbase.h>

#include "amutil.h"
#include "RuleCollection.h"

RuleCollection :: RuleCollection( DampRule* d )
{
	defaultRule = d;
    head = NULL;
    nextRuleId = 1;
}

void delList( RuleNode* r )
{
    if( !r )
        return;

    delList( r->next );
    delete r->next;
    delete r->data;
}

RuleCollection :: ~RuleCollection()
{
    delList( head );
    head = NULL;
}

void RuleCollection :: addRule( CBaseEntity* person, DampRule* d )
{
	if( !IsPlayerValid(person)  )
	{
	    LOG_MESSAGE( PLID, "ERROR: RuleCollection :: addRule -> Invalid Player, cannot add rule to collection" );
	    return;
    }
    d->id = nextRuleId;
    nextRuleId++;
    head = new RuleNode( head, d, GETPLAYERUSERID( person->edict() ) );
}

BOOL RuleCollection :: delRule( CBaseEntity* person )
{
	if( !IsPlayerValid(person)  )
	{
	    LOG_MESSAGE( PLID, "ERROR: RuleCollection :: delRule -> Invalid Player, cannot add rule to collection" );
	    return 0;
    }
    RuleNode* temp = head;
    RuleNode* prev = 0;
    while( temp )
    {
        if( temp->uid == GETPLAYERUSERID( person->edict() ) )
        {
            if( prev == 0 )
            {
                head = temp->next;
            }
            else
            {
                prev->next = temp->next;
            }
            delete temp->data;
            delete temp;
            return 1;
        }
        prev = temp;
        temp = temp->next;
    }
    LOG_MESSAGE( PLID, "ERROR: RuleCollection :: delRule -> Rule not found for player with UID %i", GETPLAYERUSERID( person->edict() ) );
    return 0;
}


DampRule* RuleCollection :: findByPerson( CBaseEntity* person )
{
	if( !IsPlayerValid(person)  )
	{
	    LOG_MESSAGE( PLID, "ERROR: RuleCollection :: findByPerson -> Invalid Player, cannot attempt find" );
	    return NULL;
    }
    RuleNode* temp = head;
    while( temp )
    {
        if( temp->uid == GETPLAYERUSERID( person->edict() ) )
            return temp->data;
        temp = temp->next;
    }
    return defaultRule;
}

DampRule* RuleCollection :: findById( int id )
{
	if( id < 0  )
	{
	    LOG_MESSAGE( PLID, "ERROR: RuleCollection :: findById -> Invalid Id (%i), cannot attempt find", id );
	    return NULL;
    }

    RuleNode* temp = head;
    while( temp )
    {
        if( temp->data->id == id )
            return temp->data;
        temp = temp->next;
    }
    return NULL;
}